package org.mb.session;

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Random;

import javax.microedition.midlet.MIDlet;

/**
 * Converations is whole time the app runs the same object instance witch
 * travels from one controller to another.
 *
 * @author gubriansky
 *
 */
public class Conversation {

	/**
	 * Conversation ID
	 */
	private static long id;

	/**
	 * this array of map holds all conversation data
	 */
	private static Hashtable[] map = new Hashtable[1];
	// private static Hashtable[] map = new Hashtable();

	/**
	 * cancel controller key
	 */
	public static final String CANCEL_CONTROLLER = "CANCEL_CONTROLLER";

	/**
	 * controller key
	 */
	public static final String CONTROLLER = "CONTROLLER";

	/**
	 * midlet key
	 */
	public static final String MIDLET = "MIDLET";

	/**
	 * initialize Conversation and sets random ID
	 *
	 * @return new Conversation
	 */
	public static Conversation init() {
		Conversation c = new Conversation();
		c.setId(new Random().nextLong());
		return c;
	}

	/**
	 * see init()
	 */
	public void clear() {
		init();
	}

	/**
	 * resize current map and adds one new map at the end of array
	 */
	public void addNewMap() {
		if (map.length >= 1) {
			int newSize = map.length + 1;
			Hashtable[] newMap = new Hashtable[newSize];
			for (int i = 0; i < map.length; i++) {
				newMap[i] = map[i];
			}
			newMap[newSize - 1] = new Hashtable();
			map = newMap;
		}
	}

	/**
	 * resize current map and substracts one map from the end of array
	 */
	public void removeLastMap() {
		if (map.length >= 1) {
			int newSize = map.length - 1;
			Hashtable[] newMap = new Hashtable[newSize];
			for (int i = 0; i < newSize; i++) {
				newMap[i] = map[i];
			}
			map = newMap;
		}
	}

	/**
	 * id getter
	 *
	 * @return long id
	 */
	public long getId() {
		return id;
	}

	/**
	 * id setter
	 *
	 * @param newId
	 */
	public void setId(long newId) {
		id = newId;
	}

	/**
	 * Set attribute to last map in map array
	 *
	 * @param key
	 * @param value
	 */
	public void setAttribute(String key, Object value) {
		System.out.println(map.length + ":" + value.toString());
		if (map.length == 1 && map[0] == null) {
			map[0] = new Hashtable();
		}
		if (value != null) {
			map[map.length - 1].put(key, value);
		}
		if (value == null && key != null) {
			map[map.length - 1].remove(key);
		}
	}

	/**
	 * Set attribute to last map in map array
	 *
	 * @param key
	 * @param value
	 */
	public void setAttribute(Class key, Object value) {
		if (value != null) {
			map[map.length - 1].put(key, value);
		}
		if (value == null && key != null) {
			map[map.length - 1].remove(key);
		}
	}

	/**
	 * Get attribute from last map in map array
	 *
	 * @param key
	 * @return
	 */
	public Object getAttribute(Class key) {
		if (map[map.length - 1].containsKey(key)) {
			return map[map.length - 1].get(key);
		}
		return null;
	}

	/**
	 * Get attribute from last map in map array
	 *
	 * @param key
	 * @return
	 */
	public Object getAttribute(String key) {
		if (map[map.length - 1].containsKey(key)) {
			return map[map.length - 1].get(key);
		}
		return null;
	}

	/**
	 * Returns cancel controller class from current conversation
	 *
	 * @return
	 */
	public Class getCancelController() {
		if (map[map.length - 1].containsKey(CANCEL_CONTROLLER)) {
			return (Class) map[map.length - 1].get(CANCEL_CONTROLLER);
		}
		return null;
	}

	/**
	 * Remove attribute from current map conversation
	 *
	 * @param formKey
	 */
	public void removeAttribute(String formKey) {
		if (map[map.length - 1].containsKey(formKey)) {
			map[map.length - 1].remove(formKey);
		}
	}

	/**
	 * return actual midlet from conversation
	 *
	 * @return
	 */
	public MIDlet getMIDlet() {
		if (map[map.length - 1].containsKey(MIDLET)) {
			return (MIDlet) map[map.length - 1].get(MIDLET);
		}
		return null;
	}

	/**
	 * return actual controller from conversation
	 *
	 * @return
	 */
	public Object getController() {
		if (map[map.length - 1].containsKey(CONTROLLER)) {
			return map[map.length - 1].get(CONTROLLER);
		}
		return null;
	}

	/**
	 * adds new map to existing conversation
	 *
	 * @param map2
	 * @param keys
	 */
	public void addAll(Hashtable map2) {
		if (map != null) {
			for(Enumeration e = map2.keys(); e.hasMoreElements();){
				String key = (String) e.nextElement();
				setAttribute(key, map2.get(key));
			}
		}
	}

}
